#include "vector3.h"


Vector3::Vector3(const Vector3& cp) : x(cp.x), y(cp.y), z(cp.z), w(1.0)
{
}

Vector3::Vector3(float X, float Y, float Z, float W) : x(X), y(Y), z(Z), w(W)
{
}

void Vector3::normalize()
{
	float l = length();
	if (l == 0.0f || l == 1.0f)
		return;

	#ifdef __SSE__
		// multiply with the reciprocal value of l
		f = _mm_mul_ps( f, _mm_rcp_ps( _mm_set_ps1(l) ) );
	#else
		l = 1.0f / l;
		x *= l;
		y *= l;
		z *= l;
	#endif
}

Vector3 cross(const Vector3& lhs, const Vector3& rhs)
{
	Vector3 result;
	#ifdef __SSE__
		__m128 lhs1 = _mm_set_ps(lhs.y, lhs.z, lhs.x, lhs.w);
		__m128 rhs1 = _mm_set_ps(rhs.z, rhs.x, rhs.y, rhs.w);
		__m128 lhs2 = _mm_set_ps(lhs.z, lhs.x, lhs.y, lhs.w);
		__m128 rhs2 = _mm_set_ps(rhs.y, rhs.z, rhs.x, rhs.w);

		result.f = _mm_sub_ps( _mm_mul_ps(lhs1, rhs1), _mm_mul_ps(lhs2, rhs2) );

	#else
		result.x = lhs.y*rhs.z - rhs.y*lhs.z;
		result.y = lhs.z*rhs.x - rhs.z*lhs.x;
		result.z = lhs.x*rhs.y - rhs.x*lhs.y;
	#endif

	return result;
}
